@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2013 Zabbix SIA
+** Copyright (C) 2001-2015 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,7 +29,6 @@ class CImportReferencer {
 	 * @var array with references to interfaceid (hostid -> reference_name -> interfaceid)
 	 */
 	public $interfacesCache = array();
-	protected $processedHosts = array();
 	protected $groups = array();
 	protected $templates = array();
 	protected $hosts = array();
@@ -37,11 +36,14 @@ class CImportReferencer {
 	protected $items = array();
 	protected $valueMaps = array();
 	protected $triggers = array();
+	protected $graphs = array();
 	protected $iconMaps = array();
 	protected $maps = array();
 	protected $screens = array();
+	protected $templateScreens = array();
 	protected $macros = array();
 	protected $proxies = array();
+	protected $hostPrototypes = array();
 	protected $groupsRefs;
 	protected $templatesRefs;
 	protected $hostsRefs;
@@ -49,34 +51,16 @@ class CImportReferencer {
 	protected $itemsRefs;
 	protected $valueMapsRefs;
 	protected $triggersRefs;
+	protected $graphsRefs;
 	protected $iconMapsRefs;
 	protected $mapsRefs;
 	protected $screensRefs;
+	protected $templateScreensRefs;
 	protected $macrosRefs;
 	protected $proxiesRefs;
+	protected $hostPrototypesRefs;
 
 
-	/**
-	 * Add host/template that has been updated or created, i.e. all items, discovery rules, etc,
-	 * related to these hosts/templates should be created or updated too.
-	 *
-	 * @param $host
-	 */
-	public function addProcessedHost($host) {
-		$this->processedHosts[$host] = $host;
-	}
-
-	/**
-	 * Checks if host/template has been created or updated during the current import.
-	 *
-	 * @param $host
-	 *
-	 * @return bool
-	 */
-	public function isProcessedHost($host) {
-		return isset($this->processedHosts[$host]);
-	}
-
 	/**
 	 * Get group id by name.
 	 *
@@ -118,6 +102,7 @@ class CImportReferencer {
 		if ($this->templatesRefs === null) {
 			$this->selectTemplates();
 		}
+
 		return isset($this->templatesRefs[$host]) ? $this->templatesRefs[$host] : false;
 	}
 
@@ -195,7 +180,7 @@ class CImportReferencer {
 	}
 
 	/**
-	 * Get trigger id by trigger name and expression.
+	 * Get trigger ID by trigger name and expression.
 	 *
 	 * @param string $name
 	 * @param string $expression
@@ -210,6 +195,22 @@ class CImportReferencer {
 		return isset($this->triggersRefs[$name][$expression]) ? $this->triggersRefs[$name][$expression] : false;
 	}
 
+	/**
+	 * Get graph ID by host ID and graph name.
+	 *
+	 * @param string $hostId
+	 * @param string $name
+	 *
+	 * @return string|bool
+	 */
+	public function resolveGraph($hostId, $name) {
+		if ($this->graphsRefs === null) {
+			$this->selectGraphs();
+		}
+
+		return isset($this->graphsRefs[$hostId][$name]) ? $this->graphsRefs[$hostId][$name] : false;
+	}
+
 	/**
 	 * Get icon map id by name.
 	 *
@@ -256,7 +257,25 @@ class CImportReferencer {
 	}
 
 	/**
-	 * Get macro id by host id and macro name.
+	 * Get templated screen ID by template ID and screen name.
+	 *
+	 * @param string $templateId
+	 * @param string $screenName
+	 *
+	 * @return string|bool
+	 */
+	public function resolveTemplateScreen($templateId, $screenName) {
+		if ($this->templateScreensRefs === null) {
+			$this->selectTemplateScreens();
+		}
+
+		return isset($this->templateScreensRefs[$templateId][$screenName])
+			? $this->templateScreensRefs[$templateId][$screenName]
+			: false;
+	}
+
+	/**
+	 * Get macro ID by host ID and macro name.
 	 *
 	 * @param string $hostid
 	 * @param string $name
@@ -286,6 +305,28 @@ class CImportReferencer {
 		return isset($this->proxiesRefs[$name]) ? $this->proxiesRefs[$name] : false;
 	}
 
+	/**
+	 * Get proxy id by name.
+	 *
+	 * @param string $hostId
+	 * @param string $discoveryRuleId
+	 * @param string $hostPrototype
+	 *
+	 * @return string|bool
+	 */
+	public function resolveHostPrototype($hostId, $discoveryRuleId, $hostPrototype) {
+		if ($this->hostPrototypesRefs === null) {
+			$this->selectHostPrototypes();
+		}
+
+		if (isset($this->hostPrototypesRefs[$hostId][$discoveryRuleId][$hostPrototype])) {
+			return $this->hostPrototypesRefs[$hostId][$discoveryRuleId][$hostPrototype];
+		}
+		else {
+			return false;
+		}
+	}
+
 	/**
 	 * Add group names that need association with a database group id.
 	 *
@@ -359,17 +400,6 @@ class CImportReferencer {
 		}
 	}
 
-	/**
-	 * Add application name association with application id.
-	 *
-	 * @param string $hostId
-	 * @param string $name
-	 * @param string $appId
-	 */
-	public function addApplicationRef($hostId, $name, $appId) {
-		$this->applicationsRefs[$hostId][$name] = $appId;
-	}
-
 	/**
 	 * Add item keys that need association with a database item id.
 	 * Input array has format:
@@ -422,6 +452,22 @@ class CImportReferencer {
 		}
 	}
 
+	/**
+	 * Add graph names that need association with a database graph ID.
+	 * Input array has format:
+	 * array('hostname1' => array('graphname1', 'graphname2'), 'hostname2' => array('graphname1'), ...)
+	 *
+	 * @param array $graphs
+	 */
+	public function addGraphs(array $graphs) {
+		foreach ($graphs as $host => $hostGraphs) {
+			if (!isset($this->graphs[$host])) {
+				$this->graphs[$host] = array();
+			}
+			$this->graphs[$host] = array_unique(array_merge($this->graphs[$host], $hostGraphs));
+		}
+	}
+
 	/**
 	 * Add trigger name/expression association with trigger id.
 	 *
@@ -470,6 +516,15 @@ class CImportReferencer {
 		$this->screens = array_unique(array_merge($this->screens, $screens));
 	}
 
+	/**
+	 * Add templated screen names that need association with a database screen id.
+	 *
+	 * @param array $screens
+	 */
+	public function addTemplateScreens(array $screens) {
+		$this->templateScreens = array_unique(array_merge($this->templateScreens, $screens));
+	}
+
 	/**
 	 * Add screen name association with screen id.
 	 *
@@ -480,6 +535,16 @@ class CImportReferencer {
 		$this->screensRefs[$name] = $screenId;
 	}
 
+	/**
+	 * Add template screen name association with template screen ID.
+	 *
+	 * @param string $screenName
+	 * @param string $templateScreenId
+	 */
+	public function addTemplateScreenRef($screenName, $templateScreenId) {
+		$this->templateScreensRefs[$screenName] = $templateScreenId;
+	}
+
 	/**
 	 * Add macros names that need association with a database macro id.
 	 *
@@ -524,6 +589,27 @@ class CImportReferencer {
 		$this->proxiesRefs[$name] = $proxyId;
 	}
 
+	/**
+	 * Add host prototypes that need association with a database host prototype id.
+	 *
+	 * @param array $hostPrototypes
+	 */
+	public function addHostPrototypes(array $hostPrototypes) {
+		foreach ($hostPrototypes as $host => $discoveryRule) {
+			if (!isset($this->hostPrototypes[$host])) {
+				$this->hostPrototypes[$host] = array();
+			}
+			foreach ($discoveryRule as $discoveryRuleKey => $hostPrototypes) {
+				if (!isset($this->hostPrototypes[$host][$discoveryRuleKey])) {
+					$this->hostPrototypes[$host][$discoveryRuleKey] = array();
+				}
+				$this->hostPrototypes[$host][$discoveryRuleKey] = array_unique(
+					array_merge($this->hostPrototypes[$host][$discoveryRuleKey], $hostPrototypes)
+				);
+			}
+		}
+	}
+
 	/**
 	 * Select group ids for previously added group names.
 	 */
@@ -551,10 +637,10 @@ class CImportReferencer {
 		if (!empty($this->templates)) {
 			$this->templatesRefs = array();
 			$dbTemplates = API::Template()->get(array(
-				'filter' => array('host' => $this->templates),
-				'output' => array('hostid', 'host'),
+				'output' => array('host', 'templateid'),
 				'preservekeys' => true,
-				'editable' => true
+				'editable' => true,
+				'filter' => array('host' => $this->templates)
 			));
 			foreach ($dbTemplates as $template) {
 				$this->templatesRefs[$template['host']] = $template['templateid'];
@@ -570,6 +656,7 @@ class CImportReferencer {
 	protected function selectHosts() {
 		if (!empty($this->hosts)) {
 			$this->hostsRefs = array();
+			// fetch only normal hosts, discovered hosts must not be imported
 			$dbHosts = API::Host()->get(array(
 				'filter' => array('host' => $this->hosts),
 				'output' => array('hostid', 'host'),
@@ -631,8 +718,8 @@ class CImportReferencer {
 			}
 
 			if ($sqlWhere) {
-				$dbitems = DBselect('SELECT i.itemid,i.hostid,i.key_ FROM items i WHERE '.implode(' OR ', $sqlWhere));
-				while ($dbItem = DBfetch($dbitems)) {
+				$dbItems = DBselect('SELECT i.itemid,i.hostid,i.key_ FROM items i WHERE '.implode(' OR ', $sqlWhere));
+				while ($dbItem = DBfetch($dbItems)) {
 					$this->itemsRefs[$dbItem['hostid']][$dbItem['key_']] = $dbItem['itemid'];
 				}
 			}
@@ -669,37 +756,56 @@ class CImportReferencer {
 		if (!empty($this->triggers)) {
 			$this->triggersRefs = array();
 
-			$triggerIds = array();
-			$sql = 'SELECT t.triggerid,t.expression,t.description'.
-				' FROM triggers t'.
-				' WHERE '.dbConditionString('t.description', array_keys($this->triggers));
-			$dbTriggers = DBselect($sql);
-			while ($dbTrigger = DBfetch($dbTriggers)) {
-				$dbExpr = explode_exp($dbTrigger['expression']);
-				foreach ($this->triggers as $name => $expressions) {
-					if ($name == $dbTrigger['description']) {
-						foreach ($expressions as $expression) {
-							if ($expression == $dbExpr) {
-								$this->triggersRefs[$name][$expression] = $dbTrigger['triggerid'];
-								$triggerIds[] = $dbTrigger['triggerid'];
-							}
-						}
-					}
+			$dbTriggers = API::Trigger()->get(array(
+				'output' => array('triggerid', 'expression', 'description'),
+				'expandExpression' => true,
+				'filter' => array(
+					'description' => array_keys($this->triggers),
+					'flags' => array(
+						ZBX_FLAG_DISCOVERY_NORMAL,
+						ZBX_FLAG_DISCOVERY_PROTOTYPE,
+						ZBX_FLAG_DISCOVERY_CREATED
+					)
+				),
+				'editable' => true
+			));
+
+			foreach ($dbTriggers as $dbTrigger) {
+				if (isset($this->triggers[$dbTrigger['description']][$dbTrigger['expression']])) {
+					$this->triggersRefs[$dbTrigger['description']][$dbTrigger['expression']] = $dbTrigger['triggerid'];
+				}
+			}
+		}
+	}
+
+	/**
+	 * Select graph IDs for previously added graph names.
+	 */
+	protected function selectGraphs() {
+		if ($this->graphs) {
+			$this->graphsRefs = array();
+
+			$graphNames = array();
+
+			foreach ($this->graphs as $graphs) {
+				foreach ($graphs as $graph) {
+					$graphNames[$graph] = $graph;
 				}
 			}
 
-			$allowedTriggers = API::Trigger()->get(array(
-				'triggerids' => $triggerIds,
-				'output' => API_OUTPUT_SHORTEN,
-				'filter' => array('flags' => array(ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_CHILD)),
-				'editable' => true,
-				'preservekeys' => true
+			$dbGraphs = API::Graph()->get(array(
+				'output' => array('graphid', 'name'),
+				'selectHosts' => array('hostid'),
+				'filter' => array(
+					'name' => $graphNames,
+					'flags' => null
+				),
+				'editable' => true
 			));
-			foreach ($this->triggersRefs as $name => $expressions) {
-				foreach ($expressions as $expression => $triggerId) {
-					if (!isset($allowedTriggers[$triggerId])) {
-						unset($this->triggersRefs[$name][$expression]);
-					}
+
+			foreach ($dbGraphs as $dbGraph) {
+				foreach ($dbGraph['hosts'] as $host) {
+					$this->graphsRefs[$host['hostid']][$dbGraph['name']] = $dbGraph['graphid'];
 				}
 			}
 		}
@@ -712,6 +818,13 @@ class CImportReferencer {
 		$this->triggersRefs = null;
 	}
 
+	/**
+	 * Unset graph refs to make referencer select them from DB again.
+	 */
+	public function refreshGraphs() {
+		$this->graphsRefs = null;
+	}
+
 	/**
 	 * Select icon map ids for previously added icon maps names.
 	 */
@@ -768,6 +881,28 @@ class CImportReferencer {
 		}
 	}
 
+	/**
+	 * Select template screen IDs for previously added screen names and template IDs.
+	 */
+	protected function selectTemplateScreens() {
+		if ($this->templateScreens) {
+			$this->templateScreensRefs = array();
+
+			$dbScreens = DBselect(
+				'SELECT s.screenid, s.name, s.templateid'.
+				' FROM screens s'.
+				' WHERE s.templateid IS NOT NULL '.
+					' AND '.dbConditionString('s.name', $this->templateScreens)
+			);
+
+			while ($dbScreen = DBfetch($dbScreens)) {
+				$this->templateScreensRefs[$dbScreen['templateid']][$dbScreen['name']] = $dbScreen['screenid'];
+			}
+
+			$this->templateScreens = array();
+		}
+	}
+
 	/**
 	 * Select macro ids for previously added macro names.
 	 */
@@ -812,4 +947,37 @@ class CImportReferencer {
 			$this->proxies = array();
 		}
 	}
+
+	/**
+	 * Select host prototype ids for previously added host prototypes names.
+	 */
+	protected function selectHostPrototypes() {
+		if (!empty($this->hostPrototypes)) {
+			$this->hostPrototypesRefs = array();
+			$sqlWhere = array();
+			foreach ($this->hostPrototypes as $host => $discoveryRule) {
+				$hostId = $this->resolveHostOrTemplate($host);
+
+				foreach ($discoveryRule as $discoveryRuleKey => $hostPrototypes) {
+					$discoveryRuleId = $this->resolveItem($hostId, $discoveryRuleKey);
+					if ($hostId) {
+						$sqlWhere[] = '(hd.parent_itemid='.zbx_dbstr($discoveryRuleId).' AND '.dbConditionString('h.host', $hostPrototypes).')';
+					}
+				}
+			}
+
+			if ($sqlWhere) {
+				$query = DBselect(
+					'SELECT h.host,h.hostid,hd.parent_itemid,i.hostid AS parent_hostid '.
+					' FROM hosts h,host_discovery hd,items i'.
+					' WHERE h.hostid=hd.hostid'.
+						' AND hd.parent_itemid=i.itemid'.
+						' AND '.implode(' OR ', $sqlWhere)
+				);
+				while ($data = DBfetch($query)) {
+					$this->hostPrototypesRefs[$data['parent_hostid']][$data['parent_itemid']][$data['host']] = $data['hostid'];
+				}
+			}
+		}
+	}
 }
