@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2015 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -198,14 +198,50 @@ $hostList->addRow(_('Visible name'), $visiblenameTB);
 
 // groups for normal hosts
 if (!$isDiscovered) {
+	$hostGroups = array_combine($hostGroups, $hostGroups);
 	$grp_tb = new CTweenBox($frmHost, 'groups', $hostGroups, 10);
-	$all_groups = API::HostGroup()->get(array(
+
+	// get user allowed host groups and sort them by name
+	$groupsAllowed = API::HostGroup()->get(array(
 		'editable' => true,
-		'output' => API_OUTPUT_EXTEND
+		'output' => API_OUTPUT_EXTEND,
+		'preservekeys' => true
 	));
-	order_result($all_groups, 'name');
-	foreach ($all_groups as $group) {
-		$grp_tb->addItem($group['groupid'], $group['name']);
+	order_result($groupsAllowed, 'name');
+
+	if (getRequest('form') === 'update') {
+		// get other host groups that user has also read permissions and sort by name
+		$all_groups = API::HostGroup()->get(array(
+			'output' => array('groupid', 'name'),
+			'preservekeys' => true
+		));
+		order_result($all_groups, 'name');
+
+		// Add existing host groups to list and, depending on permissions show name as enabled or disabled
+		$groupsInList = array();
+
+		foreach ($all_groups as $group) {
+			if (isset($hostGroups[$group['groupid']])) {
+				$grp_tb->addItem($group['groupid'], $group['name'], true, isset($groupsAllowed[$group['groupid']]));
+				$groupsInList[$group['groupid']] = true;
+			}
+		}
+
+		// Add other host groups that user has permissions to, if not yet added to the list.
+		foreach ($groupsAllowed as $group) {
+			if (!isset($groupsInList[$group['groupid']])) {
+				$grp_tb->addItem($group['groupid'], $group['name']);
+			}
+		}
+	}
+	else {
+		/*
+		 * When cloning a host or creating a new one, don't show read-only host groups in left box. Show empty or posted
+		 * groups in case of an error.
+		 */
+		foreach ($groupsAllowed as $group) {
+			$grp_tb->addItem($group['groupid'], $group['name']);
+		}
 	}
 
 	$hostList->addRow(_('Groups'), $grp_tb->get(_('In groups'), _('Other groups')));
@@ -261,7 +297,7 @@ if (!$isDiscovered) {
 	$col = new CCol($helpTextWhenDragInterfaceAgent);
 	$col->setAttribute('colspan', 6);
 	$buttonRow = new CRow(array($buttonCol, $col));
-	$buttonRow->setAttribute('id', 'agentIterfacesFooter');
+	$buttonRow->setAttribute('id', 'agentInterfacesFooter');
 
 	$ifTab->addRow($buttonRow);
 
@@ -278,7 +314,7 @@ if (!$isDiscovered) {
 	$col = new CCol($helpTextWhenDragInterfaceSNMP);
 	$col->setAttribute('colspan', 6);
 	$buttonRow = new CRow(array($buttonCol, $col));
-	$buttonRow->setAttribute('id', 'SNMPIterfacesFooter');
+	$buttonRow->setAttribute('id', 'SNMPInterfacesFooter');
 
 	$ifTab->addRow($buttonRow);
 
@@ -294,7 +330,7 @@ if (!$isDiscovered) {
 	$col = new CCol($helpTextWhenDragInterfaceJMX);
 	$col->setAttribute('colspan', 6);
 	$buttonRow = new CRow(array($buttonCol, $col));
-	$buttonRow->setAttribute('id', 'JMXIterfacesFooter');
+	$buttonRow->setAttribute('id', 'JMXInterfacesFooter');
 	$ifTab->addRow($buttonRow);
 
 	$hostList->addRow(_('JMX interfaces'), new CDiv($ifTab, 'border_dotted objectgroup inlineblock interface-group'), false, null, 'interface-row');
@@ -309,7 +345,7 @@ if (!$isDiscovered) {
 	$col = new CCol($helpTextWhenDragInterfaceIPMI);
 	$col->setAttribute('colspan', 6);
 	$buttonRow = new CRow(array($buttonCol, $col));
-	$buttonRow->setAttribute('id', 'IPMIIterfacesFooter');
+	$buttonRow->setAttribute('id', 'IPMIInterfacesFooter');
 
 	$ifTab->addRow($buttonRow);
 	$hostList->addRow(_('IPMI interfaces'), new CDiv($ifTab, 'border_dotted objectgroup inlineblock interface-group'), false, null, 'interface-row');
@@ -323,7 +359,7 @@ else {
 		$existingInterfaceTypes[$interface['type']] = true;
 		$interfaces[$interface['interfaceid']] = $interface;
 	}
-	zbx_add_post_js('hostInterfacesManager.add('.CJs::encodeJson($interfaces).');');
+	zbx_add_post_js('hostInterfacesManager.add('.CJs::encodeJson(array_values($interfaces)).');');
 	zbx_add_post_js('hostInterfacesManager.disable();');
 
 	// table for agent interfaces with footer
@@ -342,7 +378,7 @@ else {
 		new CCol(SPACE, 'interface-control')
 	));
 
-	$row = new CRow(null, null, 'agentIterfacesFooter');
+	$row = new CRow(null, null, 'agentInterfacesFooter');
 	if (!isset($existingInterfaceTypes[INTERFACE_TYPE_AGENT])) {
 		$row->addItem(new CCol(null, 'interface-drag-control'));
 		$row->addItem(new CCol(_('No agent interfaces found.'), null, 5));
@@ -356,7 +392,7 @@ else {
 	$ifTab->setAttribute('id', 'SNMPInterfaces');
 	$ifTab->setAttribute('data-type', 'snmp');
 
-	$row = new CRow(null, null, 'SNMPIterfacesFooter');
+	$row = new CRow(null, null, 'SNMPInterfacesFooter');
 	if (!isset($existingInterfaceTypes[INTERFACE_TYPE_SNMP])) {
 		$row->addItem(new CCol(null, 'interface-drag-control'));
 		$row->addItem(new CCol(_('No SNMP interfaces found.'), null, 5));
@@ -369,7 +405,7 @@ else {
 	$ifTab->setAttribute('id', 'JMXInterfaces');
 	$ifTab->setAttribute('data-type', 'jmx');
 
-	$row = new CRow(null, null, 'JMXIterfacesFooter');
+	$row = new CRow(null, null, 'JMXInterfacesFooter');
 	if (!isset($existingInterfaceTypes[INTERFACE_TYPE_JMX])) {
 		$row->addItem(new CCol(null, 'interface-drag-control'));
 		$row->addItem(new CCol(_('No JMX interfaces found.'), null, 5));
@@ -382,7 +418,7 @@ else {
 	$ifTab->setAttribute('id', 'IPMIInterfaces');
 	$ifTab->setAttribute('data-type', 'ipmi');
 
-	$row = new CRow(null, null, 'IPMIIterfacesFooter');
+	$row = new CRow(null, null, 'IPMIInterfacesFooter');
 	if (!isset($existingInterfaceTypes[INTERFACE_TYPE_IPMI])) {
 		$row->addItem(new CCol(null, 'interface-drag-control'));
 		$row->addItem(new CCol(_('No IPMI interfaces found.'), null, 5));
