@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2013 Zabbix SIA
+** Copyright (C) 2001-2015 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,29 +34,33 @@ require_once dirname(__FILE__).'/include/page_header.php';
 $fields = array(
 	'groupid' =>			array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null),
 	'hostid' =>				array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		null),
-	'triggerid' =>			array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'(isset({form})&&({form}=="update"))'),
-	'copy_type' =>			array(T_ZBX_INT, O_OPT, P_SYS,	IN('0,1'),	'isset({copy})'),
+	'triggerid' =>			array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'(isset({form}) && ({form} == "update"))'),
+	'copy_type' =>			array(T_ZBX_INT, O_OPT, P_SYS,	IN(array(COPY_TYPE_TO_HOST, COPY_TYPE_TO_TEMPLATE, COPY_TYPE_TO_HOST_GROUP)), 'isset({copy})'),
 	'copy_mode' =>			array(T_ZBX_INT, O_OPT, P_SYS,	IN('0'),	null),
 	'type' =>				array(T_ZBX_INT, O_OPT, null,	IN('0,1'),	null),
-	'description' =>		array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({save})', _('Name')),
-	'expression' =>			array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({save})', _('Expression')),
-	'priority' =>			array(T_ZBX_INT, O_OPT, null,	IN('0,1,2,3,4,5'), 'isset({save})'),
-	'comments' =>			array(T_ZBX_STR, O_OPT, null,	null,		'isset({save})'),
-	'url' =>				array(T_ZBX_STR, O_OPT, null,	null,		'isset({save})'),
+	'description' =>		array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})', _('Name')),
+	'expression' =>			array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})', _('Expression')),
+	'priority' =>			array(T_ZBX_INT, O_OPT, null,	IN('0,1,2,3,4,5'), 'isset({add}) || isset({update})'),
+	'comments' =>			array(T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'),
+	'url' =>				array(T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'),
 	'status' =>				array(T_ZBX_STR, O_OPT, null,	null,		null),
 	'input_method' =>		array(T_ZBX_INT, O_OPT, null,	NOT_EMPTY,	'isset({toggle_input_method})'),
-	'expr_temp' =>			array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'(isset({add_expression})||isset({and_expression})||isset({or_expression})||isset({replace_expression}))', _('Expression')),
-	'expr_target_single' => array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'(isset({and_expression})||isset({or_expression})||isset({replace_expression}))'),
+	'expr_temp' =>			array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'(isset({add_expression}) || isset({and_expression}) || isset({or_expression}) || isset({replace_expression}))', _('Expression')),
+	'expr_target_single' => array(T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'(isset({and_expression}) || isset({or_expression}) || isset({replace_expression}))'),
 	'dependencies' =>		array(T_ZBX_INT, O_OPT, null,	DB_ID,		null),
 	'new_dependency' =>		array(T_ZBX_INT, O_OPT, null,	DB_ID.'{}>0', 'isset({add_dependency})'),
 	'g_triggerid' =>		array(T_ZBX_INT, O_OPT, null,	DB_ID,		null),
 	'copy_targetid' =>		array(T_ZBX_INT, O_OPT, null,	DB_ID,		null),
-	'filter_groupid' =>		array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'isset({copy})&&(isset({copy_type})&&({copy_type}==0))'),
+	'copy_groupid' =>		array(T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'isset({copy}) && (isset({copy_type}) && {copy_type} == 0)'),
 	'showdisabled' =>		array(T_ZBX_INT, O_OPT, P_SYS,	IN('0,1'),	null),
-	'massupdate' =>			array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
 	'visible' =>			array(T_ZBX_STR, O_OPT, null,	null,		null),
 	// actions
-	'go' =>					array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
+	'action' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT,
+								IN('"trigger.masscopyto","trigger.massdelete","trigger.massdisable",'.
+									'"trigger.massenable","trigger.massupdate","trigger.massupdateform"'
+								),
+								null
+							),
 	'toggle_input_method' =>array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
 	'add_expression' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
 	'and_expression' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
@@ -70,78 +74,98 @@ $fields = array(
 	'group_delete' =>		array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
 	'copy' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
 	'clone' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'save' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
-	'mass_save' =>			array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
+	'add' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
+	'update' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
+	'massupdate' =>			array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
 	'delete' =>				array(T_ZBX_STR, O_OPT, P_SYS|P_ACT, null,	null),
 	'cancel' =>				array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
 	'form' =>				array(T_ZBX_STR, O_OPT, P_SYS,	null,		null),
-	'form_refresh' =>		array(T_ZBX_INT, O_OPT, null,	null,		null)
+	'form_refresh' =>		array(T_ZBX_INT, O_OPT, null,	null,		null),
+	// sort and sortorder
+	'sort' =>				array(T_ZBX_STR, O_OPT, P_SYS, IN('"description","priority","status"'),		null),
+	'sortorder' =>			array(T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null)
 );
-$_REQUEST['showdisabled'] = get_request('showdisabled', CProfile::get('web.triggers.showdisabled', 1));
+$_REQUEST['showdisabled'] = getRequest('showdisabled', CProfile::get('web.triggers.showdisabled', 1));
 
 check_fields($fields);
-validate_sort_and_sortorder('description', ZBX_SORT_UP);
 
 $_REQUEST['status'] = isset($_REQUEST['status']) ? TRIGGER_STATUS_ENABLED : TRIGGER_STATUS_DISABLED;
 $_REQUEST['type'] = isset($_REQUEST['type']) ? TRIGGER_MULT_EVENT_ENABLED : TRIGGER_MULT_EVENT_DISABLED;
-$_REQUEST['go'] = get_request('go', 'none');
 
 // validate permissions
-if (get_request('triggerid')) {
+$triggerIds = getRequest('g_triggerid', array());
+if (!is_array($triggerIds)) {
+	$triggerIds = zbx_toArray($triggerIds);
+}
+
+$triggerId = getRequest('triggerid');
+if ($triggerId !== null) {
+	$triggerIds[] = $triggerId;
+}
+
+if ($triggerIds) {
 	$triggers = API::Trigger()->get(array(
-		'triggerids' => $_REQUEST['triggerid'],
+		'triggerids' => array_keys(array_flip($triggerIds)),
 		'output' => array('triggerid'),
 		'preservekeys' => true,
 		'filter' => array('flags' => ZBX_FLAG_DISCOVERY_NORMAL),
 		'editable' => true
 	));
-	if (!$triggers) {
+	if ($triggers) {
+		foreach ($triggerIds as $triggerId) {
+			if (!isset($triggers[$triggerId])) {
+				access_deny();
+			}
+		}
+	}
+	else {
 		access_deny();
 	}
 }
-if (get_request('hostid') && !API::Host()->isWritable(array($_REQUEST['hostid']))) {
+if (getRequest('hostid') && !API::Host()->isWritable(array($_REQUEST['hostid']))) {
 	access_deny();
 }
 /*
  * Actions
  */
+$exprAction = null;
 if (isset($_REQUEST['add_expression'])) {
 	$_REQUEST['expression'] = $_REQUEST['expr_temp'];
 	$_REQUEST['expr_temp'] = '';
 }
 elseif (isset($_REQUEST['and_expression'])) {
-	$_REQUEST['expr_action'] = '&';
+	$exprAction = 'and';
 }
 elseif (isset($_REQUEST['or_expression'])) {
-	$_REQUEST['expr_action'] = '|';
+	$exprAction = 'or';
 }
 elseif (isset($_REQUEST['replace_expression'])) {
-	$_REQUEST['expr_action'] = 'r';
+	$exprAction = 'r';
 }
-elseif (isset($_REQUEST['remove_expression']) && zbx_strlen($_REQUEST['remove_expression'])) {
-	$_REQUEST['expr_action'] = 'R';
+elseif (getRequest('remove_expression')) {
+	$exprAction = 'R';
 	$_REQUEST['expr_target_single'] = $_REQUEST['remove_expression'];
 }
 elseif (isset($_REQUEST['clone']) && isset($_REQUEST['triggerid'])) {
 	unset($_REQUEST['triggerid']);
 	$_REQUEST['form'] = 'clone';
 }
-elseif (isset($_REQUEST['save'])) {
+elseif (hasRequest('add') || hasRequest('update')) {
 	$trigger = array(
-		'expression' => $_REQUEST['expression'],
-		'description' => $_REQUEST['description'],
-		'priority' => $_REQUEST['priority'],
-		'status' => $_REQUEST['status'],
-		'type' => $_REQUEST['type'],
-		'comments' => $_REQUEST['comments'],
-		'url' => $_REQUEST['url'],
-		'dependencies' => zbx_toObject(get_request('dependencies', array()), 'triggerid')
+		'expression' => getRequest('expression'),
+		'description' => getRequest('description'),
+		'priority' => getRequest('priority'),
+		'status' => getRequest('status'),
+		'type' => getRequest('type'),
+		'comments' => getRequest('comments'),
+		'url' => getRequest('url'),
+		'dependencies' => zbx_toObject(getRequest('dependencies', array()), 'triggerid')
 	);
 
-	if (get_request('triggerid')) {
+	if (hasRequest('update')) {
 		// update only changed fields
 		$oldTrigger = API::Trigger()->get(array(
-			'triggerids' => $_REQUEST['triggerid'],
+			'triggerids' => getRequest('triggerid'),
 			'output' => API_OUTPUT_EXTEND,
 			'selectDependencies' => array('triggerid')
 		));
@@ -158,7 +182,7 @@ elseif (isset($_REQUEST['save'])) {
 		unset($oldTrigger['dependencies']);
 
 		$triggerToUpdate = array_diff_assoc($trigger, $oldTrigger);
-		$triggerToUpdate['triggerid'] = $_REQUEST['triggerid'];
+		$triggerToUpdate['triggerid'] = getRequest('triggerid');
 
 		// dependencies
 		$updateDepencencies = false;
@@ -185,21 +209,20 @@ elseif (isset($_REQUEST['save'])) {
 
 	if ($result) {
 		unset($_REQUEST['form']);
-		clearCookies($result, $_REQUEST['hostid']);
+		uncheckTableRows(getRequest('hostid'));
 	}
 }
 elseif (isset($_REQUEST['delete']) && isset($_REQUEST['triggerid'])) {
 	DBstart();
 
-	$result = API::Trigger()->delete($_REQUEST['triggerid']);
+	$result = API::Trigger()->delete(array(getRequest('triggerid')));
 	$result = DBend($result);
 
-	show_messages($result, _('Trigger deleted'), _('Cannot delete trigger'));
-	clearCookies($result, $_REQUEST['hostid']);
-
 	if ($result) {
 		unset($_REQUEST['form'], $_REQUEST['triggerid']);
+		uncheckTableRows(getRequest('hostid'));
 	}
+	show_messages($result, _('Trigger deleted'), _('Cannot delete trigger'));
 }
 elseif (isset($_REQUEST['add_dependency']) && isset($_REQUEST['new_dependency'])) {
 	if (!isset($_REQUEST['dependencies'])) {
@@ -211,19 +234,19 @@ elseif (isset($_REQUEST['add_dependency']) && isset($_REQUEST['new_dependency'])
 		}
 	}
 }
-elseif ($_REQUEST['go'] == 'massupdate' && isset($_REQUEST['mass_save']) && isset($_REQUEST['g_triggerid'])) {
-	$visible = get_request('visible', array());
+elseif (hasRequest('action') && getRequest('action') == 'trigger.massupdate' && hasRequest('massupdate') && hasRequest('g_triggerid')) {
+	$visible = getRequest('visible', array());
 
 	// update triggers
 	$triggersToUpdate = array();
-	foreach ($_REQUEST['g_triggerid'] as $triggerid) {
+	foreach (getRequest('g_triggerid') as $triggerid) {
 		$trigger = array('triggerid' => $triggerid);
 
 		if (isset($visible['priority'])) {
-			$trigger['priority'] = get_request('priority');
+			$trigger['priority'] = getRequest('priority');
 		}
 		if (isset($visible['dependencies'])) {
-			$trigger['dependencies'] = zbx_toObject(get_request('dependencies', array()), 'triggerid');
+			$trigger['dependencies'] = zbx_toObject(getRequest('dependencies', array()), 'triggerid');
 		}
 
 		$triggersToUpdate[] = $trigger;
@@ -234,15 +257,16 @@ elseif ($_REQUEST['go'] == 'massupdate' && isset($_REQUEST['mass_save']) && isse
 	$result = API::Trigger()->update($triggersToUpdate);
 	$result = DBend($result);
 
-	show_messages($result, _('Trigger updated'), _('Cannot update trigger'));
-	clearCookies($result, $_REQUEST['hostid']);
-
 	if ($result) {
-		unset($_REQUEST['massupdate'], $_REQUEST['form'], $_REQUEST['g_triggerid']);
+		unset($_REQUEST['form'], $_REQUEST['g_triggerid']);
+		uncheckTableRows(getRequest('hostid'));
 	}
+	show_messages($result, _('Trigger updated'), _('Cannot update trigger'));
 }
-elseif (str_in_array($_REQUEST['go'], array('activate', 'disable')) && isset($_REQUEST['g_triggerid'])) {
-	$status = (getRequest('go') == 'activate') ? TRIGGER_STATUS_ENABLED : TRIGGER_STATUS_DISABLED;
+elseif (hasRequest('action') && str_in_array(getRequest('action'), array('trigger.massenable', 'trigger.massdisable')) && hasRequest('g_triggerid')) {
+	$enable = (getRequest('action') == 'trigger.massenable');
+	$status = $enable ? TRIGGER_STATUS_ENABLED : TRIGGER_STATUS_DISABLED;
+	$update = array();
 
 	// get requested triggers with permission check
 	$dbTriggers = API::Trigger()->get(array(
@@ -252,31 +276,43 @@ elseif (str_in_array($_REQUEST['go'], array('activate', 'disable')) && isset($_R
 	));
 
 	if ($dbTriggers) {
-		$updateTriggers = array();
 		foreach ($dbTriggers as $dbTrigger) {
-			$updateTriggers[] = array(
+			$update[] = array(
 				'triggerid' => $dbTrigger['triggerid'],
 				'status' => $status
 			);
 		}
 
-		$goResult = API::Trigger()->update($updateTriggers);
+		$result = API::Trigger()->update($update);
 	}
 	else {
-		$goResult = true;
+		$result = true;
 	}
 
-	show_messages($goResult, _('Status updated'), _('Cannot update status'));
-	clearCookies($goResult, getRequest('hostid'));
+	$updated = count($update);
+	$messageSuccess = $enable
+		? _n('Trigger enabled', 'Triggers enabled', $updated)
+		: _n('Trigger disabled', 'Triggers disabled', $updated);
+	$messageFailed = $enable
+		? _n('Cannot enable trigger', 'Cannot enable triggers', $updated)
+		: _n('Cannot disable trigger', 'Cannot disable triggers', $updated);
+
+	if ($result) {
+		uncheckTableRows(getRequest('hostid'));
+		unset($_REQUEST['g_triggerid']);
+	}
+	show_messages($result, $messageSuccess, $messageFailed);
 }
-elseif ($_REQUEST['go'] == 'copy_to' && isset($_REQUEST['copy']) && isset($_REQUEST['g_triggerid'])) {
-	if (isset($_REQUEST['copy_targetid']) && $_REQUEST['copy_targetid'] > 0 && isset($_REQUEST['copy_type'])) {
-		if ($_REQUEST['copy_type'] == 0) { // hosts
-			$hosts_ids = $_REQUEST['copy_targetid'];
+elseif (hasRequest('action') && getRequest('action') == 'trigger.masscopyto' && hasRequest('copy') && hasRequest('g_triggerid')) {
+	if (hasRequest('copy_targetid') && getRequest('copy_targetid') > 0 && hasRequest('copy_type')) {
+		// hosts or templates
+		if (getRequest('copy_type') == COPY_TYPE_TO_HOST || getRequest('copy_type') == COPY_TYPE_TO_TEMPLATE) {
+			$hosts_ids = getRequest('copy_targetid');
 		}
-		else { // groups
+		// host groups
+		else {
 			$hosts_ids = array();
-			$group_ids = $_REQUEST['copy_targetid'];
+			$group_ids = getRequest('copy_targetid');
 
 			$db_hosts = DBselect(
 				'SELECT DISTINCT h.hostid'.
@@ -291,73 +327,83 @@ elseif ($_REQUEST['go'] == 'copy_to' && isset($_REQUEST['copy']) && isset($_REQU
 
 		DBstart();
 
-		$goResult = copyTriggersToHosts($_REQUEST['g_triggerid'], $hosts_ids, get_request('hostid'));
-		$goResult = DBend($goResult);
+		$result = copyTriggersToHosts(getRequest('g_triggerid'), $hosts_ids, getRequest('hostid'));
+		$result = DBend($result);
 
-		show_messages($goResult, _('Trigger added'), _('Cannot add trigger'));
-		clearCookies($goResult, $_REQUEST['hostid']);
-
-		$_REQUEST['go'] = 'none2';
+		if ($result) {
+			uncheckTableRows(getRequest('hostid'));
+			unset($_REQUEST['g_triggerid']);
+		}
+		show_messages($result, _('Trigger added'), _('Cannot add trigger'));
 	}
 	else {
 		show_error_message(_('No target selected'));
 	}
 }
-elseif ($_REQUEST['go'] == 'delete' && isset($_REQUEST['g_triggerid'])) {
-	$goResult = API::Trigger()->delete($_REQUEST['g_triggerid']);
+elseif (hasRequest('action') && getRequest('action') == 'trigger.massdelete' && hasRequest('g_triggerid')) {
+	$result = API::Trigger()->delete(getRequest('g_triggerid'));
 
-	show_messages($goResult, _('Triggers deleted'), _('Cannot delete triggers'));
-	clearCookies($goResult, $_REQUEST['hostid']);
+	if ($result) {
+		uncheckTableRows(getRequest('hostid'));
+	}
+	show_messages($result, _('Triggers deleted'), _('Cannot delete triggers'));
 }
 
 /*
  * Display
  */
-if ($_REQUEST['go'] == 'massupdate' && isset($_REQUEST['g_triggerid'])) {
-	$triggersView = new CView('configuration.triggers.massupdate', getTriggerMassupdateFormData());
+if (hasRequest('action') && getRequest('action') == 'trigger.massupdateform' && hasRequest('g_triggerid')) {
+	$data = getTriggerMassupdateFormData();
+	$data['action'] = 'trigger.massupdate';
+	$triggersView = new CView('configuration.triggers.massupdate', $data);
 	$triggersView->render();
 	$triggersView->show();
 }
 elseif (isset($_REQUEST['form'])) {
-	$triggersView = new CView('configuration.triggers.edit', getTriggerFormData());
+	$triggersView = new CView('configuration.triggers.edit', getTriggerFormData($exprAction));
 	$triggersView->render();
 	$triggersView->show();
 }
-elseif ($_REQUEST['go'] == 'copy_to' && isset($_REQUEST['g_triggerid'])) {
-	$triggersView = new CView('configuration.copy.elements', getCopyElementsFormData('g_triggerid', _('CONFIGURATION OF TRIGGERS')));
+elseif (hasRequest('action') && getRequest('action') == 'trigger.masscopyto' && hasRequest('g_triggerid')) {
+	$data = getCopyElementsFormData('g_triggerid', _('CONFIGURATION OF TRIGGERS'));
+	$data['action'] = 'trigger.masscopyto';
+	$triggersView = new CView('configuration.copy.elements', $data);
 	$triggersView->render();
 	$triggersView->show();
 }
 else {
+	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'description'));
+	$sortOrder = getRequest('sortorder', CProfile::get('web.'.$page['file'].'.sortorder', ZBX_SORT_UP));
+
+	CProfile::update('web.'.$page['file'].'.sort', $sortField, PROFILE_TYPE_STR);
+	CProfile::update('web.'.$page['file'].'.sortorder', $sortOrder, PROFILE_TYPE_STR);
+
 	$data = array(
-		'showdisabled' => get_request('showdisabled', 1),
+		'showdisabled' => getRequest('showdisabled', 1),
 		'parent_discoveryid' => null,
 		'triggers' => array(),
-		'displayNodes' => (is_array(get_current_nodeid()) && empty($_REQUEST['groupid']) && empty($_REQUEST['hostid']))
+		'sort' => $sortField,
+		'sortorder' => $sortOrder
 	);
+
 	CProfile::update('web.triggers.showdisabled', $data['showdisabled'], PROFILE_TYPE_INT);
 
 	$data['pageFilter'] = new CPageFilter(array(
-		'groups' => array('not_proxy_hosts' => true, 'editable' => true),
+		'groups' => array('with_hosts_and_templates' => true, 'editable' => true),
 		'hosts' => array('templated_hosts' => true, 'editable' => true),
 		'triggers' => array('editable' => true),
-		'groupid' => get_request('groupid', null),
-		'hostid' => get_request('hostid', null),
-		'triggerid' => get_request('triggerid', null)
+		'groupid' => getRequest('groupid'),
+		'hostid' => getRequest('hostid')
 	));
-	if ($data['pageFilter']->triggerid > 0) {
-		$data['triggerid'] = $data['pageFilter']->triggerid;
-	}
 	$data['groupid'] = $data['pageFilter']->groupid;
 	$data['hostid'] = $data['pageFilter']->hostid;
 
 	// get triggers
-	$sortfield = getPageSortField('description');
 	if ($data['pageFilter']->hostsSelected) {
 		$options = array(
 			'editable' => true,
 			'output' => array('triggerid'),
-			'sortfield' => $sortfield,
+			'sortfield' => $sortField,
 			'limit' => $config['search_limit'] + 1
 		);
 		if (empty($data['showdisabled'])) {
@@ -372,10 +418,10 @@ else {
 		$data['triggers'] = API::Trigger()->get($options);
 	}
 
-	$_REQUEST['hostid'] = get_request('hostid', $data['pageFilter']->hostid);
+	$_REQUEST['hostid'] = getRequest('hostid', $data['pageFilter']->hostid);
 
 	// paging
-	$data['paging'] = getPagingLine($data['triggers'], array('triggerid'), array('hostid' => $_REQUEST['hostid']));
+	$data['paging'] = getPagingLine($data['triggers']);
 
 	$data['triggers'] = API::Trigger()->get(array(
 		'triggerids' => zbx_objectValues($data['triggers'], 'triggerid'),
@@ -383,33 +429,64 @@ else {
 		'selectHosts' => API_OUTPUT_EXTEND,
 		'selectItems' => array('itemid', 'hostid', 'key_', 'type', 'flags', 'status'),
 		'selectFunctions' => API_OUTPUT_EXTEND,
-		'selectDependencies' => API_OUTPUT_EXTEND,
+		'selectDependencies' => array('triggerid', 'description'),
 		'selectDiscoveryRule' => API_OUTPUT_EXTEND
 	));
-	order_result($data['triggers'], $sortfield, getPageSortOrder());
+
+	if ($sortField === 'status') {
+		orderTriggersByStatus($data['triggers'], $sortOrder);
+	}
+	else {
+		order_result($data['triggers'], $sortField, $sortOrder);
+	}
+
+	$dependencyIds = array();
+	foreach ($data['triggers'] as $trigger) {
+		foreach ($trigger['dependencies'] as $depTrigger) {
+			$dependencyIds[$depTrigger['triggerid']] = $depTrigger['triggerid'];
+		}
+	}
+
+	$dependencyTriggers = array();
+	if ($dependencyIds) {
+		$dependencyTriggers = API::Trigger()->get(array(
+			'triggerids' => $dependencyIds,
+			'output' => array('triggerid', 'flags', 'description', 'status'),
+			'selectHosts' => array('hostid', 'name'),
+			'preservekeys' => true
+		));
+
+		// sort dependencies
+		foreach ($data['triggers'] as &$trigger) {
+			if (count($trigger['dependencies']) > 1) {
+				order_result($trigger['dependencies'], 'description', ZBX_SORT_UP);
+			}
+		}
+		unset($trigger);
+
+		// sort dependency trigger hosts
+		foreach ($dependencyTriggers as &$trigger) {
+			if (count($dependencyTriggers[$trigger['triggerid']]['hosts']) > 1) {
+				order_result($dependencyTriggers[$trigger['triggerid']]['hosts'], 'name', ZBX_SORT_UP);
+			}
+		}
+		unset($trigger);
+	}
+
+	$data['dependencyTriggers'] = $dependencyTriggers;
 
 	// get real hosts
 	$data['realHosts'] = getParentHostsByTriggers($data['triggers']);
 
 	// determine, show or not column of errors
-	$data['showErrorColumn'] = true;
 	if ($data['hostid'] > 0) {
-		$host = API::Host()->get(array(
-			'hostids' => $_REQUEST['hostid'],
-			'output' => array('status'),
-			'templated_hosts' => true,
-			'editable' => true
+		$data['showInfoColumn'] = (bool) API::Host()->get(array(
+			'hostids' => $data['hostid'],
+			'output' => array('status')
 		));
-		$host = reset($host);
-		$data['showErrorColumn'] = (!$host || $host['status'] != HOST_STATUS_TEMPLATE);
 	}
-
-	// nodes
-	if ($data['displayNodes']) {
-		foreach ($data['triggers'] as &$trigger) {
-			$trigger['nodename'] = get_node_name_by_elid($trigger['triggerid'], true);
-		}
-		unset($trigger);
+	else {
+		$data['showInfoColumn'] = true;
 	}
 
 	// render view
