@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2014 Zabbix SIA
+** Copyright (C) 2001-2015 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -20,30 +20,14 @@
 #include "common.h"
 #include "sysinfo.h"
 
+#include <tlhelp32.h>
+
 #include "symbols.h"
 #include "log.h"
 
 #define MAX_PROCESSES	4096
 #define MAX_NAME	256
 
-/* function 'zbx_get_processname' require 'baseName' with size 'MAX_NAME' */
-static int	zbx_get_processname(HANDLE hProcess, char *baseName)
-{
-	HMODULE	hMod;
-	DWORD	dwSize;
-	wchar_t	name[MAX_NAME];
-
-	if (0 == EnumProcessModules(hProcess, &hMod, sizeof(hMod), &dwSize))
-		return FAIL;
-
-	if (0 == GetModuleBaseName(hProcess, hMod, name, MAX_NAME))
-		return FAIL;
-
-	zbx_unicode_to_utf8_static(name, baseName, MAX_NAME);
-
-	return SUCCEED;
-}
-
 /* function 'zbx_get_process_username' require 'userName' with size 'MAX_NAME' */
 static int	zbx_get_process_username(HANDLE hProcess, char *userName)
 {
@@ -53,8 +37,6 @@ static int	zbx_get_process_username(HANDLE hProcess, char *userName)
 	wchar_t		name[MAX_NAME], dom[MAX_NAME];
 	int		iUse, res = FAIL;
 
-	assert(userName);
-
 	/* clean result; */
 	*userName = '\0';
 
@@ -93,11 +75,10 @@ lbl_err:
 
 int	PROC_NUM(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
-	HANDLE	hProcess;
-	DWORD	procList[MAX_PROCESSES], dwSize;
-	int	i, proccount, max_proc_cnt,
-		proc_ok = 0,
-		user_ok = 0;
+	HANDLE	hProcessSnap, hProcess;
+	PROCESSENTRY32	pe32;
+	int	proccount,
+		proc_ok;
 	char	*procName,
 		*userName,
 		baseName[MAX_PATH],
@@ -112,46 +93,55 @@ int	PROC_NUM(AGENT_REQUEST *request, AGENT_RESULT *result)
 	procName = get_rparam(request, 0);
 	userName = get_rparam(request, 1);
 
-	if (0 == EnumProcesses(procList, sizeof(DWORD) * MAX_PROCESSES, &dwSize))
+	if (INVALID_HANDLE_VALUE == (hProcessSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0)))
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain system information."));
 		return SYSINFO_RET_FAIL;
 	}
 
-	max_proc_cnt = dwSize / sizeof(DWORD);
+	pe32.dwSize = sizeof(PROCESSENTRY32);
+
+	if (FALSE == Process32First(hProcessSnap, &pe32))
+	{
+		CloseHandle(hProcessSnap);
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain system information."));
+		return SYSINFO_RET_FAIL;
+	}
+
 	proccount = 0;
 
-	for (i = 0; i < max_proc_cnt; i++)
+	do
 	{
-		proc_ok = 0;
-		user_ok = 0;
+		proc_ok = 1;
 
-		if (NULL != (hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, procList[i])))
+		if (NULL != procName && '\0' != *procName)
 		{
-			if (NULL != procName && '\0' != *procName)
-			{
-				if (SUCCEED == zbx_get_processname(hProcess, baseName))
-					if (0 == stricmp(baseName, procName))
-						proc_ok = 1;
-			}
-			else
-				proc_ok = 1;
+			zbx_unicode_to_utf8_static(pe32.szExeFile, baseName, MAX_NAME);
 
-			if (0 != proc_ok && NULL != userName && '\0' != *userName)
-			{
-				if (SUCCEED == zbx_get_process_username(hProcess, uname))
-					if (0 == stricmp(uname, userName))
-						user_ok = 1;
-			}
-			else
-				user_ok = 1;
-
-			if (0 != user_ok && 0 != proc_ok)
-				proccount++;
-
-			CloseHandle(hProcess);
+			if (0 != stricmp(baseName, procName))
+				proc_ok = 0;
 		}
+
+		if (0 != proc_ok && NULL != userName && '\0' != *userName)
+		{
+			hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, pe32.th32ProcessID);
+
+			if (NULL == hProcess || SUCCEED != zbx_get_process_username(hProcess, uname) ||
+					0 != stricmp(uname, userName))
+			{
+				proc_ok = 0;
+			}
+
+			if (NULL != hProcess)
+				CloseHandle(hProcess);
+		}
+
+		if (0 != proc_ok)
+			proccount++;
 	}
+	while (TRUE == Process32Next(hProcessSnap, &pe32));
+
+	CloseHandle(hProcessSnap);
 
 	SET_UI64_RESULT(result, proccount);
 
@@ -302,14 +292,14 @@ static int	GetProcessAttribute(HANDLE hProcess,int attr,int type,int count,doubl
 
 int	PROC_INFO(AGENT_REQUEST *request, AGENT_RESULT *result)
 {
-	DWORD		*procList, dwSize;
-	HANDLE		hProcess;
+	HANDLE		hProcessSnap, hProcess;
+	PROCESSENTRY32	pe32;
 	char		*proc_name, *attr, *type, baseName[MAX_PATH];
 	const char	*attrList[] = {"vmsize", "wkset", "pf", "ktime", "utime", "gdiobj", "userobj", "io_read_b", "io_read_op",
 					"io_write_b", "io_write_op", "io_other_b", "io_other_op", NULL},
 			*typeList[] = {"min", "max", "avg", "sum", NULL};
 	double		value;
-	int		i, proc_cnt, counter, attr_id, type_id, ret = SYSINFO_RET_OK;
+	int		counter, attr_id, type_id, ret = SYSINFO_RET_OK;
 
 	if (3 < request->nparam)
 	{
@@ -363,31 +353,48 @@ int	PROC_INFO(AGENT_REQUEST *request, AGENT_RESULT *result)
 		return SYSINFO_RET_FAIL;
 	}
 
-	procList = (DWORD *)malloc(MAX_PROCESSES * sizeof(DWORD));
-
-	if (0 == EnumProcesses(procList, sizeof(DWORD) * MAX_PROCESSES, &dwSize))
+	if (INVALID_HANDLE_VALUE == (hProcessSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0)))
 	{
 		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain system information."));
 		return SYSINFO_RET_FAIL;
 	}
 
-	proc_cnt = dwSize / sizeof(DWORD);
+	pe32.dwSize = sizeof(PROCESSENTRY32);
+
+	if (FALSE == Process32First(hProcessSnap, &pe32))
+	{
+		CloseHandle(hProcessSnap);
+		SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain system information."));
+		return SYSINFO_RET_FAIL;
+	}
+
 	counter = 0;
 	value = 0;
 
-	for (i = 0; i < proc_cnt; i++)
+	do
 	{
-		if (NULL != (hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,FALSE, procList[i])))
+		zbx_unicode_to_utf8_static(pe32.szExeFile, baseName, MAX_NAME);
+
+		if (0 == stricmp(baseName, proc_name))
 		{
-			if (SUCCEED == zbx_get_processname(hProcess, baseName))
-				if (0 == stricmp(baseName, proc_name))
-					if (SYSINFO_RET_OK != (ret = GetProcessAttribute(hProcess, attr_id, type_id, counter++, &value)))
-						break;
-			CloseHandle(hProcess);
+			if (NULL != (hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE,
+					pe32.th32ProcessID)))
+			{
+				ret = GetProcessAttribute(hProcess, attr_id, type_id, counter++, &value);
+
+				CloseHandle(hProcess);
+
+				if (SYSINFO_RET_OK != ret)
+				{
+					SET_MSG_RESULT(result, zbx_strdup(NULL, "Cannot obtain process information."));
+					break;
+				}
+			}
 		}
 	}
+	while (TRUE == Process32Next(hProcessSnap, &pe32));
 
-	free(procList);
+	CloseHandle(hProcessSnap);
 
 	if (SYSINFO_RET_OK == ret)
 		SET_DBL_RESULT(result, value);
